﻿//*********************************************************
//
// Copyright(C) Seiko Epson Corporation 2019. All rights reserved.
//
//*********************************************************

using System.Threading.Tasks;

namespace SampleBrightnessControl.Communications
{
    internal class BrightnessCommunication : SerialCommunication
    {
        public async Task<bool> SetBrightnessAsync(string brightness)
        {
            if (CheckBrightnessLevel(brightness))
            {
                return "OK" == await WriteAndReadAsync("setbright " + brightness).ConfigureAwait(false);
            }
            return false;
        }

        public async Task<bool> SetBrightnessAutoAsync()
        {
            return "OK" == await WriteAndReadAsync("setbright 50").ConfigureAwait(false);
        }

        public async Task<string> GetBrightnessAsync()
        {
            var readData = await WriteAndReadAsync("getbright").ConfigureAwait(false);
            if (CheckBrightnessLevel(readData))
            {
                return readData;
            }

            if (readData.Equals("50"))
            {
                return "Auto";
            }

            return string.Empty;
        }

        private bool CheckBrightnessLevel(string brightness)
        {
            if (int.TryParse(brightness, out int level))
            {
                return 0 <= level && level <= 20;
            }
            return false;
        }
    }
}
