﻿//*********************************************************
//
// Copyright(C) Seiko Epson Corporation 2019. All rights reserved.
//
//*********************************************************

using SampleBrightnessControl.Communications;
using SampleBrightnessControl.Watchers;
using System;
using Windows.ApplicationModel;
using Windows.Foundation;
using Windows.UI.ViewManagement;
using Windows.UI.Xaml;
using Windows.UI.Xaml.Controls;
using Windows.UI.Xaml.Navigation;


namespace SampleBrightnessControl
{
    public sealed partial class MainPage : Page
    {
        private readonly MoverioWatcher moverioWatcher = new MoverioWatcher();
        private readonly BrightnessCommunication brightnessCommunication = new BrightnessCommunication();
        private string currentDeviceInformationId = string.Empty;

        public MainPage()
        {
            InitializeComponent();
            ApplicationView.GetForCurrentView().SetPreferredMinSize(new Size(540, 260));
            ApplicationView.PreferredLaunchViewSize = new Size(540, 260);
            ApplicationView.PreferredLaunchWindowingMode = ApplicationViewWindowingMode.PreferredLaunchViewSize;
        }

        protected override void OnNavigatedTo(NavigationEventArgs e)
        {
            Application.Current.Suspending += Current_Suspending;
            Application.Current.Resuming += Current_Resuming;
            StartCommunication();
        }

        protected override void OnNavigatingFrom(NavigatingCancelEventArgs e)
        {
            Application.Current.Suspending -= Current_Suspending;
            Application.Current.Resuming -= Current_Resuming;
            ClosedCommunication();
        }

        private void Current_Resuming(object sender, object e)
        {
            StartCommunication();
        }

        private void Current_Suspending(object sender, SuspendingEventArgs e)
        {
            ClosedCommunication();
        }

        private void StartCommunication()
        {
            moverioWatcher.Add += MoverioWatcher_AddAsync;
            moverioWatcher.Remove += MoverioWatcher_RemoveAsync;
            moverioWatcher.Start();
        }

        private void ClosedCommunication()
        {
            moverioWatcher.Stop();
            moverioWatcher.Add -= MoverioWatcher_AddAsync;
            moverioWatcher.Remove -= MoverioWatcher_RemoveAsync;
            brightnessCommunication.Close();
            currentDeviceInformationId = string.Empty;
            EnableButtons(false);
        }

        private async void MoverioWatcher_RemoveAsync(object sender, MoverioInfo e)
        {
            if (currentDeviceInformationId.Equals(e.id))
            {
                brightnessCommunication.Close();
                currentDeviceInformationId = string.Empty;
                await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, () => EnableButtons(false));
            }
        }

        private async void MoverioWatcher_AddAsync(object sender, MoverioInfo e)
        {
            if (string.IsNullOrEmpty(currentDeviceInformationId))
            {
                currentDeviceInformationId = e.id;
                if (await brightnessCommunication.OpenAsync(currentDeviceInformationId).ConfigureAwait(false))
                {
                    await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, () =>
                    {
                        SetMoverioTypeDisplay(e.type);
                        EnableButtons(true);
                    });
                }
            }
        }

        private async void SetBrightness_ClickAsync(object sender, RoutedEventArgs _)
        {
            var button = sender as Button;
            var brightness = button.Content as string;

            if (await brightnessCommunication.SetBrightnessAsync(brightness).ConfigureAwait(false))
            {
                await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, () =>
                {
                    EnableButtons(true);
                    button.IsEnabled = false;
                });
            }
        }

        private async void SetBrightnessAuto_ClickAsync(object sender, RoutedEventArgs _)
        {
            var button = sender as Button;

            if (await brightnessCommunication.SetBrightnessAutoAsync().ConfigureAwait(false))
            {
                await Dispatcher.RunAsync(Windows.UI.Core.CoreDispatcherPriority.Normal, () =>
                {
                    EnableButtons(true);
                    button.IsEnabled = false;
                });
            }
        }

        private void GetBrightness_Click(object sender, RoutedEventArgs _)
        {
            var brightness = brightnessCommunication.GetBrightnessAsync().Result;
            if (string.IsNullOrEmpty(brightness))
            {
                CurrentBrightness.Text = "Error";
            }
            else
            {
                CurrentBrightness.Text = brightness;
            }
        }

        private void SetMoverioTypeDisplay(MoverioType type)
        {
            ButtonAuto.Visibility = (MoverioType.BT35E == type || MoverioType.BT40 == type) ? Visibility.Visible : Visibility.Collapsed;
        }

        private void EnableButtons(bool enable)
        {
            ButtonGet.IsEnabled = enable;
            ButtonLevel0.IsEnabled = enable;
            ButtonLevel1.IsEnabled = enable;
            ButtonLevel2.IsEnabled = enable;
            ButtonLevel3.IsEnabled = enable;
            ButtonLevel4.IsEnabled = enable;
            ButtonLevel5.IsEnabled = enable;
            ButtonLevel6.IsEnabled = enable;
            ButtonLevel7.IsEnabled = enable;
            ButtonLevel8.IsEnabled = enable;
            ButtonLevel9.IsEnabled = enable;
            ButtonLevel10.IsEnabled = enable;
            ButtonLevel11.IsEnabled = enable;
            ButtonLevel12.IsEnabled = enable;
            ButtonLevel13.IsEnabled = enable;
            ButtonLevel14.IsEnabled = enable;
            ButtonLevel15.IsEnabled = enable;
            ButtonLevel16.IsEnabled = enable;
            ButtonLevel17.IsEnabled = enable;
            ButtonLevel18.IsEnabled = enable;
            ButtonLevel19.IsEnabled = enable;
            ButtonLevel20.IsEnabled = enable;
            ButtonAuto.IsEnabled = enable;
        }
    }
}
