﻿//*********************************************************
//
// Copyright(C) Seiko Epson Corporation 2019. All rights reserved.
//
//*********************************************************

using System;
using System.Collections.Generic;
using Windows.Devices.Enumeration;
using Windows.Devices.SerialCommunication;

namespace SampleBrightnessControl.Watchers
{
    internal class MoverioWatcher
    {
        private readonly Dictionary<string, MoverioType> SupportMoverios = new Dictionary<string, MoverioType>()
        {
            { "VID_0483&PID_5750" , MoverioType.BT35E},
            { "VID_04B8&PID_0C0C" , MoverioType.BT30C},
            { "VID_04B8&PID_0D12" , MoverioType.BT40},
            { "VID_04B8&PID_0C0E" , MoverioType.BT45C}
        };

        private readonly DeviceWatcher deviceWatcher;
        public event EventHandler<MoverioInfo> Add;
        public event EventHandler<MoverioInfo> Remove;

        public MoverioWatcher()
        {
            var deviceSelector = SerialDevice.GetDeviceSelector();

            deviceWatcher = DeviceInformation.CreateWatcher(deviceSelector);
        }

        private void OnDeviceRemoved(DeviceWatcher sender, DeviceInformationUpdate args)
        {
            foreach (var key in SupportMoverios.Keys)
            {
                if (args.Id.Contains(key))
                {
                    OnRemove(new MoverioInfo(SupportMoverios[key], args.Id));
                }
            }
        }

        private void OnDeviceAdded(DeviceWatcher sender, DeviceInformation args)
        {
            foreach (var key in SupportMoverios.Keys)
            {
                if (args.Id.Contains(key))
                {
                    OnAdd(new MoverioInfo(SupportMoverios[key], args.Id));
                }
            }
        }

        public void Start()
        {
            if ((deviceWatcher.Status != DeviceWatcherStatus.Started) && (deviceWatcher.Status != DeviceWatcherStatus.EnumerationCompleted))
            {
                deviceWatcher.Added += OnDeviceAdded;
                deviceWatcher.Removed += OnDeviceRemoved;
                deviceWatcher.Start();
            }
        }

        public void Stop()
        {
            if (deviceWatcher == null)
            {
                return;
            }

            switch (deviceWatcher.Status)
            {
                case DeviceWatcherStatus.Started:
                case DeviceWatcherStatus.EnumerationCompleted:
                    deviceWatcher.Added -= OnDeviceAdded;
                    deviceWatcher.Removed -= OnDeviceRemoved;
                    deviceWatcher.Stop();
                    break;
            }
        }

        protected virtual void OnAdd(MoverioInfo e)
        {
            Add?.Invoke(this, e);
        }

        protected virtual void OnRemove(MoverioInfo e)
        {
            Remove?.Invoke(this, e);
        }
    }
}
