﻿//*********************************************************
//
// Copyright(C) Seiko Epson Corporation 2021. All rights reserved.
//
//*********************************************************

using System.Threading.Tasks;

namespace SampleDisplayControl.Communications
{
    internal class DisplayCommunication : SerialCommunication
    {
        private readonly string Mode2D = "2D";
        private readonly string Mode3D = "3D";

        private readonly string ModeON = "ON";
        private readonly string ModeOFF = "OFF";

        private readonly string Mode2DValue = "0";
        private readonly string Mode3DValue = "1";

        private readonly string ModeONValue = "1";
        private readonly string ModeOFFValue = "0";

        public async Task<bool> SetModeAsync(string mode)
        {
            if (mode == Mode2D)
            {
                return await WriteAndReadAsync("set2d3d " + Mode2DValue).ConfigureAwait(false) == "OK";
            }
            else if (mode == Mode3D)
            {
                return await WriteAndReadAsync("set2d3d " + Mode3DValue).ConfigureAwait(false) == "OK";
            }
            else
            {
                return false;
            }
        }

        public async Task<string> GetModeAsync()
        {
            var mode = await WriteAndReadAsync("get2d3d").ConfigureAwait(false);

            if (mode == Mode2DValue)
            {
                return Mode2D;
            }
            else if (mode == Mode3DValue)
            {
                return Mode3D;
            }
            else
            {
                return string.Empty;
            }
        }

        public async Task<bool> SetMuteAsync(string mode)
        {
            if (mode == ModeON)
            {
                return await WriteAndReadAsync("setmute " + ModeONValue).ConfigureAwait(false) == "OK";
            }
            else if (mode == ModeOFF)
            {
                return await WriteAndReadAsync("setmute " + ModeOFFValue).ConfigureAwait(false) == "OK";
            }
            else
            {
                return false;
            }
        }

        public async Task<string> GetMuteAsync()
        {
            var mode = await WriteAndReadAsync("getmute").ConfigureAwait(false);

            if (mode == ModeONValue)
            {
                return ModeON;
            }
            else if (mode == ModeOFFValue)
            {
                return ModeOFF;
            }
            else
            {
                return string.Empty;
            }
        }

        public async Task<bool> SetAutoSleepAsync(string mode)
        {
            if (mode == ModeON)
            {
                return await WriteAndReadAsync("enableautosleep " + ModeONValue).ConfigureAwait(false) == "OK";
            }
            else if (mode == ModeOFF)
            {
                return await WriteAndReadAsync("enableautosleep " + ModeOFFValue).ConfigureAwait(false) == "OK";
            }
            else
            {
                return false;
            }
        }

        public async Task<string> GetAutoSleepAsync()
        {
            var mode = await WriteAndReadAsync("getautosleep").ConfigureAwait(false);

            if (mode == ModeONValue)
            {
                return ModeON;
            }
            else if (mode == ModeOFFValue)
            {
                return ModeOFF;
            }
            else
            {
                return string.Empty;
            }
        }

        public async Task<bool> SetManualSleepAsync(string mode)
        {
            if (mode == ModeON)
            {
                return await WriteAndReadAsync("enableusersleep " + ModeONValue).ConfigureAwait(false) == "OK";
            }
            else if (mode == ModeOFF)
            {
                return await WriteAndReadAsync("enableusersleep " + ModeOFFValue).ConfigureAwait(false) == "OK";
            }
            else
            {
                return false;
            }
        }

        public async Task<string> GetManualSleepAsync()
        {
            var mode = await WriteAndReadAsync("getusersleep").ConfigureAwait(false);

            if (mode == ModeONValue)
            {
                return ModeON;
            }
            else if (mode == ModeOFFValue)
            {
                return ModeOFF;
            }
            else
            {
                return string.Empty;
            }
        }

        public async Task<bool> SetDisplayPixcellShiftAsync(int value)
        {
            return await WriteAndReadAsync("setdisplaydistance " + value).ConfigureAwait(false) == "OK";
        }

        public async Task<string> GetDisplayPixcellShiftAsync()
        {
            var distance = await WriteAndReadAsync("getdisplaydistance").ConfigureAwait(false);

            bool result = int.TryParse(distance, out int value);
            if (result)
            {
                return distance;
            }
            else
            {
                return string.Empty;
            }
        }
    }
}
