﻿//*********************************************************
//
// Copyright(C) Seiko Epson Corporation 2019. All rights reserved.
//
//*********************************************************

using SampleSensorDataChart.Watchers;
using System;
using System.Collections.ObjectModel;
using System.Threading.Tasks;
using Windows.Devices.Sensors;
using Windows.UI.Core;
using Windows.UI.Xaml.Controls;

namespace SampleSensorDataChart.Sensors
{
    public sealed partial class AmbientLightControl : UserControl, ISensor
    {
        private readonly int keepRecords = 200;
        private readonly AmbientLightWatcher watcher = new AmbientLightWatcher();
        private LightSensor sensor;

        public ObservableCollection<SensorData> ValueIlluminanceInLux { get; } = new ObservableCollection<SensorData>();

        public AmbientLightControl()
        {
            InitializeComponent();
        }

        public void Active()
        {
            ClearValues();
            watcher.Add += Watcher_AddAsync;
            watcher.Remove += Watcher_Remove;
            watcher.Start();
        }

        public void Inactive()
        {
            watcher.Stop();
            watcher.Add -= Watcher_AddAsync;
            watcher.Remove -= Watcher_Remove;

            Disable();
        }

        private async void Watcher_AddAsync(object sender, string deviceId)
        {
            await EnableAsync(deviceId).ConfigureAwait(false);
        }

        private void Watcher_Remove(object sender, string deviceId)
        {
            Disable();
        }

        private async Task EnableAsync(string deviceId)
        {
            if (sensor != null)
            {
                Disable();
            }

            sensor = await LightSensor.FromIdAsync(deviceId);
            sensor.ReadingChanged += Sensor_ReadingChangedAsync;
        }

        private void Disable()
        {
            if (sensor == null)
            {
                return;
            }

            sensor.ReadingChanged -= Sensor_ReadingChangedAsync;
            sensor = null;
        }

        private async void Sensor_ReadingChangedAsync(LightSensor sender, LightSensorReadingChangedEventArgs args)
        {
            await Dispatcher.RunAsync(CoreDispatcherPriority.Normal, () =>
            {
                AxisIlluminance.Text = args.Reading.IlluminanceInLux.ToString();

                AddValues(ValueIlluminanceInLux, args.Reading.IlluminanceInLux);
            });
        }

        private void AddValues(ObservableCollection<SensorData> values, double value)
        {
            values.Add(new SensorData() { Value = value });
            if (values.Count > keepRecords)
            {
                values.RemoveAt(0);
            }
        }

        private void ClearValues()
        {
            ValueIlluminanceInLux.Clear();

            AxisIlluminance.Text = string.Empty;
        }
    }
}
