﻿//*********************************************************
//
// Copyright(C) Seiko Epson Corporation 2021. All rights reserved.
//
//*********************************************************

namespace SampleDisplayControl.Communications
{
    internal class DisplayControlCommunication : SerialCommunication
    {
        private readonly string Mode2D = "2D";
        private readonly string Mode3D = "3D";

        private readonly string ModeON = "ON";
        private readonly string ModeOFF = "OFF";

        private readonly string Mode2DValue = "0";
        private readonly string Mode3DValue = "1";

        private readonly string ModeONValue = "1";
        private readonly string ModeOFFValue = "0";

        public bool SetMode(string mode)
        {
            if (mode == Mode2D)
            {
                return WriteAndRead("set2d3d " + Mode2DValue) == "OK";
            }
            else if (mode == Mode3D)
            {
                return WriteAndRead("set2d3d " + Mode3DValue) == "OK";
            }
            else
            {
                return false;
            }
        }

        public string GetMode()
        {
            var mode = WriteAndRead("get2d3d");

            if (mode == Mode2DValue)
            {
                return Mode2D;
            }
            else if (mode == Mode3DValue)
            {
                return Mode3D;
            }
            else
            {
                return string.Empty;
            }
        }

        public bool SetMute(string mode)
        {
            if (mode == ModeON)
            {
                return WriteAndRead("setmute " + ModeONValue) == "OK";
            }
            else if (mode == ModeOFF)
            {
                return WriteAndRead("setmute " + ModeOFFValue) == "OK";
            }
            else
            {
                return false;
            }
        }

        public string GetMute()
        {
            var mode = WriteAndRead("getmute");

            if (mode == ModeONValue)
            {
                return ModeON;
            }
            else if (mode == ModeOFFValue)
            {
                return ModeOFF;
            }
            else
            {
                return string.Empty;
            }
        }

        public bool SetAutoSleep(string mode)
        {
            if (mode == ModeON)
            {
                return WriteAndRead("enableautosleep " + ModeONValue) == "OK";
            }
            else if (mode == ModeOFF)
            {
                return WriteAndRead("enableautosleep " + ModeOFFValue) == "OK";
            }
            else
            {
                return false;
            }
        }

        public string GetAutoSleep()
        {
            var mode = WriteAndRead("getautosleep");

            if (mode == ModeONValue)
            {
                return ModeON;
            }
            else if (mode == ModeOFFValue)
            {
                return ModeOFF;
            }
            else
            {
                return string.Empty;
            }
        }

        public bool SetManualSleep(string mode)
        {
            if (mode == ModeON)
            {
                return WriteAndRead("enableusersleep " + ModeONValue) == "OK";
            }
            else if (mode == ModeOFF)
            {
                return WriteAndRead("enableusersleep " + ModeOFFValue) == "OK";
            }
            else
            {
                return false;
            }
        }

        public string GetManualSleep()
        {
            var mode = WriteAndRead("getusersleep");

            if (mode == ModeONValue)
            {
                return ModeON;
            }
            else if (mode == ModeOFFValue)
            {
                return ModeOFF;
            }
            else
            {
                return string.Empty;
            }
        }

        public bool SetPixcellShift(int value)
        {
            return WriteAndRead("setdisplaydistance " + value) == "OK";
        }

        public string GetPixcellShift()
        {
            var distance = WriteAndRead("getdisplaydistance");

            bool result = int.TryParse(distance, out int value);
            if (result)
            {
                return distance;
            }
            else
            {
                return string.Empty;
            }
        }
    }
}
